// EXTERNAL
// -------------------------------------------------------------
function serverCmdRequestNPCategories(%client) {
	if(!%client.isSuperAdmin) {
		error(%client.name @ " requested pref info but is not a super admin!");
		return;  // no
	}
	
	// send the client the category list.
	%catList = PrefsList.categories; // lol cats
	
	for(%i = 0; %i < getFieldCount(%catList); %i++) {
		%catData = getField(%catList, %i);
		commandToClient(%client, 'NPListCategory', %i, %catData, getPrefCategoryIcon(%catData));
	}
}

function serverCmdGetCategoryPrefs(%client, %catID) {
	if(!%client.isSuperAdmin) {
		error(%client.name @ " requested pref info but is not a super admin!");
		return;  // no
	}
	
	// which one was this again?
	%catList = PrefsList.categories; // lol cats
	%category = getField(%catList, %catID);
	
	if(%category $= "") {
		echo("Invalid category ID " @ %catID @ " from " @ %client.name @ ", sending 0");
		serverCmdGetCategoryPrefs(%client, 0);
	}
	
	// send the client pref data for every pref in this category
	%numPrefs = PrefsList.getCount();
	
	for(%i = 0; %i < %numPrefs; %i++) {
		%pref = PrefsList.getObject(%i);
		%cat = %pref.category;
		
		if(%pref.hostOnly && !(getNumKeyId() == %client.bl_id || %client.bl_id == 999999)) {
			continue; // only show host-only prefs to the host
		}
		
		if(%cat $= %category) {
			if(getFirstWord(%pref.type) $= "datablock")
				%var = nameToId(getGlobalByName(%pref.variable));
			else
				%var = getGlobalByName(%pref.variable);
			
			commandToClient(%client, 'NPListPref', %i, %pref.name, %pref.subcategory, %pref.type, %pref.variableDefault, %pref.folder, %pref.restartRequired, %var);
		}
	}
}

function serverCmdPrefUpdates(%client, %show) {
	if(!%client.isSuperAdmin) {
		error(%client.name @ " is sending pref updates but is not a super admin!");
		return;  // no
	}
	
	if(%client.prefStrikes > 5) {
		return;
	}
	
	PrefsList.showUpdates = %show;
	
	if(getSimTime() - %client.lastPrefUpdate <= 500) {
		%client.prefStrikes++;
	} else {
		%client.prefStrikes = 0;
	}
	
	%client.lastPrefUpdate = getSimTime();
	
	if(%client.prefStrikes > 5) {
		messageAll('MsgAdminForce', "\c3" @ %client.name SPC "\c0thinks it's funny to spam the \"Apply\" button in the prefs menu.");
		schedule(2000, 0, prefsGetDunkedOn, %client);
		return;
	}
	
	messageAll('MsgAdminForce', "\c3" @ %client.name SPC "\c0updated the server prefs.");
}

function serverCmdPrefUpdate(%client, %data) {
	if(!%client.isSuperAdmin) {
		error(%client.name @ " is sending pref updates but is not a super admin!");
		return;  // no
	}
	
	%wc = getWordCount(%data);
	%pref = PrefsList.getObject(getFirstWord(%data));
	%val = getWords(%data, 1, %wc-1);
	
	if(%pref.hostOnly && getNumKeyID() != %client.BL_ID) {
		return; // fix for people being able to update host only values without being the host
	}
	
	// fix for prefs being "updated" just because you opened their menu/clicked in their box
	%type = %pref.type;
	
	if(getFirstWord(%type) $= "string" || getFirstWord(%type) $= "wordlist" || getFirstWord(%type) $= "BL_IDlist" || getFirstWord(%type) $= "datablocklist") {
		%curVal = getGlobalByName(%pref.variable);
		
		if(%curVal $= %val) {
			return;
		}
	}
	else if(getFirstWord(%type) $= "datablock") {
		%curVal = nameToId(getGlobalByName(%pref.variable));
		
		if(%curVal == %val) {
			return;
		}
	}
	else {
		%curVal = getGlobalByName(%pref.variable);
		
		if(%curVal == %val) {
			return;
		}
	}
	
	// actual pref??
	if(!isObject(%pref)) {
		error(%client.name @ " sent a pref update for a pref that doesn't exist!");
		return;  // no
	}
	
	if(getFirstWord(%type) $= "datablock") {
		setGlobalByName(%pref.variable, %val.getName());
	}
	else {
		setGlobalByName(%pref.variable, %val);
	}
	
	call(%pref.callback);
	
	if(PrefsList.showUpdates) {
		if(!%pref.isSecret)
			messageAll('', "+ \c3" @ %pref.category @ "/" @ %pref.subcategory @ "/" @ %pref.name SPC "\c0is now \c3" @ prefUiValue(%pref));
		else {
			messageAll('', "+ \c3" @ %pref.category @ "/" @ %pref.subcategory @ "/" @ %pref.name SPC "\c0was changed");
		}
	}
}

function serverCmdDoButtonPref(%client, %id) {
	if(!%client.isSuperAdmin) {
		error(%client.name @ " is sending pref updates but is not a super admin!");
		return;  // no
	}
	
	%pref = PrefsList.getObject(%id);
	setGlobalByName(%pref.variable, true); // fixes support for "check this box to so and so" things
	
	call(%pref.callback);
}

function serverCmdPopulateBL_IDListPlayers(%client) {
	if(!%client.isSuperAdmin) {
		error(%client.name @ " is sending pref updates but is not a super admin!");
		return;  // no
	}

	for(%i = 0; %i < $Pref::Server::Prefs::bl_idRecords; %i++) {
		%field = $Pref::Server::Prefs::bl_idRecord[%i+1];
		
		%wc = getWordCount(%field);
		
		%bl_id = getWord(%field, 0);
		%name = getWords(%field, 1, %wc-1);
		
		//echo(%field);
		
		commandToClient(%client, 'BL_IDListPlayer', %name, %bl_id);
	}
}