// INTERNAL
// -------------------------------------------------------------

// Init function.
function initPrefs() {
	if(!isObject(PrefsList))
		new ScriptGroup(PrefsList) { name = "PrefsList"; };
	
	updatePrefCategories();
}

initPrefs(); // do this right away.

// Registration.
function registerPref(%name, %category, %subcategory, %variable, %type, %folder, %default, %restartRequired, %hostOnly, %callback, %secret) {
	echo("Registering pref " @ %category @ ":" @ %name);
	
	// defaults
	if(%name $= "") {
		error("Prefs must be named.");
		return;
	}
	
	if(%category $= "" || %variable $= "" || %type $= "") {
		error("Unable to complete registration of " @ %category @ ":" @ %name @ "; Required pref values are missing.");
		return;
	}
	
	if(%default $= "") {
		if(firstWord(%type) $= "string") {
			%default = "";
		}
		else {
			%default = 0;
		}
	}
	
	if(%restartRequired $= "") {
		%restartRequired = false;
	}
	
	if(%hostOnly $= "") {
		%hostOnly = false;
	}
	
	if(%secret $= "") {
		%secret = false;
	}
	
	if(%subcategory $= "") {
		%subcategory = "General";
	}
	
	// check if this pref already exists?
	if(isPref(%name, %category)) {
		error("Unable to complete registration of " @ %category @ ":" @ %name @ "; Pref already exists.");
		return;
	}
	
	// registration
	%data = new ScriptObject()
	{
		// IMPORTANT
		name = %name;
		category = %category;
		variable = %variable;
		type = %type;
		
		// USEFUL
		subcategory = %subcategory; // dictates where we add dividers
		callback = %callback;
		hostOnly = %hostOnly;
		variableDefault = %default; // naming this variable "default" causes a syntax error because "default" is a keyword in TS.
		isSecret = %secret;
		
		// UNIMPORTANT (kept for legacy reasons)
		folder = %folder;
		restartRequired = %restartReqired;
	};
	
	PrefsList.add(%data);
	
	// does the variable defined here actually exist?
	if(getGlobalByName(%data.variable) $= "" && getFirstWord(%data.type) !$= "string") {
		setGlobalByName(%data.variable, %data.variableDefault);
	}
	
	updatePrefCategories();
}

// existance
function isPref(%name, %category) {
	if(%name $= "")
		return;
	
	if(%category $= "") {
		// search all prefs for this name.
		%numPrefs = PrefsList.getCount();
	
		for(%i = 0; %i < %numPrefs; %i++) {
			%pref = PrefsList.getObject(%i);
			
			%nam = %pref.name;
			//%cat = %pref.category;
			
			if(%nam $= %name)
				return true;
		}
	}
	else {
		// same for this but consider category too.
		%numPrefs = PrefsList.getCount();
	
		for(%i = 0; %i < %numPrefs; %i++) {
			%pref = PrefsList.getObject(%i);
			
			%nam = %pref.name;
			%cat = %pref.category;
			
			if(%nam $= %name && %cat $= %category)
				return true;
		}
	}
	
	return false;
}

// Housekeeping.
function updatePrefCategories() {
	%fieldList = "";
	
	// since I'd rather not have to generate these every time they're needed.
	%numPrefs = PrefsList.getCount();
	
	for(%i = 0; %i < %numPrefs; %i++) {
		%pref = PrefsList.getObject(%i);
		%cat = %pref.category;
		
		%already = false;
		
		for(%j = 0; %j < getFieldCount(%fieldList); %j++) {
			// did we already find this category?
			if(getField(%fieldList, %j) $= %cat) {
				%already = true;
			}
		}
		
		if(!%already) {
			%fieldList = setField(%fieldList, getFieldCount(%fieldList), %cat);
		}
	}
	
	if(%fieldList !$= PrefsList.categories) {
		echo("There are now " @ getFieldCount(%fieldList) @ " pref categories.");
		PrefsList.categories = %fieldList;
	}
}

function setPrefCategoryIcon(%cat, %filePath) {
	PrefsList.icon[%cat] = %filePath;
}

function getPrefCategoryIcon(%cat, %filePath) {
	return PrefsList.icon[%cat];
}

// Get a pref's visible value.
// When we display "this has changed to this" we don't want to be displaying some list ID or a box state.
function prefUiValue(%pref) {
	%type = %pref.type;
	%prefValue = getGlobalByName(%pref.variable);
	
	// if it's a list, we gotta do weird things
	if(firstWord(%type) $= "list") {
		%optionCount = (getWordCount(%type) - 1) / 2;
		
		for(%i = 0; %i < %optionCount; %i++) {
			%first = (%i * 2) + 1;
			
			%name = getWord(%type, %first);
			%value = getWord(%type, %first+1);
			
			if(%value == %prefValue) {
				return %name;
			}
		}
	}
	else if(firstWord(%type) $= "playerlist") {
		%s = %prefValue == 1 ? "" : "s";
		
		return %prefValue SPC "player" @ %s;
	}
	else if(firstWord(%type) $= "wordlist" || firstWord(%type) $= "BL_IDlist" || firstWord(%type) $= "datablocklist") {
		// replace _ with space in type
		%delimiter = getWord(%type, 1);
		
		if(%delimiter $= "_") {
			%delimiter = " ";
		}
		
		return "\"" @ strreplace(%prefValue, %delimiter, ";") @ "\"";
	}
	else if(firstWord(%type) $= "bool") {
		%s = %prefValue ? "Enabled" : "Disabled";
		
		return %s;
	}
	else if(firstWord(%type) $= "string") {
		return "\"" @ %prefValue @ "\"";
	}
	else {
		return %prefValue;
	}
}

function prefsGetDunkedOn(%client) {
	%msgListSize = 0;
	%msgList[%msgListSize++] = "It's not. Stop.";
	%msgList[%msgListSize++] = "I'm warning you.";
	%msgList[%msgListSize++] = "I'm not sure how people like this get SA.";
	%msgList[%msgListSize++] = "He's right. It's just hilarious.";
	%msgList[%msgListSize++] = "Maybe there should be a timeout or something.";
	
	%msgIndex = getRandom(1, 5);
	
	%msg = %msgList[%msgIndex];
	
	messageAll('MsgAdminForce', "\c0" @ %msg);
	%client.prefStrikes = 0;
}